﻿// © 2021 ABBYY Development Inc.
// SAMPLES code is property of ABBYY, exclusive rights are reserved. 
//
// DEVELOPER is allowed to incorporate SAMPLES into his own APPLICATION and modify it under 
// the  terms of  License Agreement between  ABBYY and DEVELOPER.


// ABBYY FineReader Engine 12 Sample

using System;
using System.IO;
using FREngine;

namespace Sample
{
    class Processor
    {
        Sample.EngineLoader engineLoader = null;
        private void loadEngine()
        {
            if (engineLoader == null)
            {
                engineLoader = new Sample.EngineLoader();
            }
        }

        private void loadProfile()
        {
            engineLoader.Engine.LoadPredefinedProfile("DocumentConversion_Accuracy");
            // Possible profile names are:
            //   "DocumentConversion_Accuracy", "DocumentConversion_Speed",
            //   "DocumentArchiving_Accuracy", "DocumentArchiving_Speed",
            //   "BookArchiving_Accuracy", "BookArchiving_Speed",
            //   "TextExtraction_Accuracy", "TextExtraction_Speed",
            //   "FieldLevelRecognition",
            //   "BarcodeRecognition_Accuracy", "BarcodeRecognition_Speed",
            //   "HighCompressedImageOnlyPdf",
            //   "BusinessCardsProcessing",
            //   "EngineeringDrawingsProcessing",
            //   "Version9Compatibility",
            //   "MachineReadableZone"
            //   "Default"
        }

        private void unloadEngine()
        {
            if (engineLoader != null)
            {
                engineLoader.Dispose();
                engineLoader = null;
            }
        }

        private void setupFREngine()
        {
            Console.WriteLine("Loading predefined profile...");
            loadProfile();
        }

        private void processImage()
        {
            string imagePath = Path.Combine(FreConfig.GetSamplesFolder(), @"SampleImages\Demo.tif");

            // Don't recognize PDF file with a textual content, just copy it
            if (engineLoader.Engine.IsPdfWithTextualContent(imagePath, null))
            {
                Console.WriteLine("Copy results...");
                string resultPath = Path.Combine(FreConfig.GetSamplesFolder(),
                    @"SampleImages\Demo_copy.pdf");
                File.Copy(imagePath, resultPath, true);
                return;
            }

            // Create document
            using (FRDocument document = engineLoader.Engine.CreateFRDocument())
            {
                try {
                    document.OnPageProcessed += FREngineEvents.CFRDocumentEvents.OnPageProcessed;
                    document.OnWarning += FREngineEvents.CFRDocumentEvents.OnWarning;

                    // Add image file to document
                    Console.WriteLine("Loading image...");
                    document.AddImageFile(imagePath, null, null);

                    // Recognize document
                    Console.WriteLine("Recognizing...");
                    document.Process(null);

                    // Save results
                    Console.WriteLine("Saving results...");

                    // Save results to rtf with default parameters
                    document.Export(Path.Combine(FreConfig.GetSamplesFolder(), @"SampleImages\Demo.rtf"),
                        FileExportFormatEnum.FEF_RTF, null);

                    // Save results to pdf using 'balanced' scenario
                    using (PDFExportParams pdfParams = engineLoader.Engine.CreatePDFExportParams())
                    {
                        pdfParams.Scenario = PDFExportScenarioEnum.PES_Balanced;
                        document.Export(Path.Combine(FreConfig.GetSamplesFolder(), @"SampleImages\Demo.pdf"),
                            FileExportFormatEnum.FEF_PDF, pdfParams);
                    }

                    document.OnPageProcessed -= FREngineEvents.CFRDocumentEvents.OnPageProcessed;
                    document.OnWarning -= FREngineEvents.CFRDocumentEvents.OnWarning;
                }
                catch (Exception error)
                {
                    Console.WriteLine("---Message---");
                    Console.WriteLine(error.Message);
                    Console.WriteLine("---Stack Trace---");
                    Console.WriteLine(error.StackTrace);
                }
            }
        }

        private void processWithEngine()
        {
            try
            {
                // Setup FREngine
                setupFREngine();

                // Process sample image
                processImage();
            }
            catch (Exception error)
            {
                Console.WriteLine("---Message---");
                Console.WriteLine(error.Message);
                Console.WriteLine("---Stack Trace---");
                Console.WriteLine(error.StackTrace);
            }
        }

        public void Process()
        {
            try
            {
                // Load ABBYY FineReader Engine
                Console.WriteLine("Initializing Engine...");
                loadEngine();

                // Process with ABBYY FineReader Engine
                processWithEngine();

                // Unload ABBYY FineReader Engine
                Console.WriteLine("Deinitializing Engine...");
                unloadEngine();
            }
            catch (Exception error)
            {
                Console.WriteLine("---Message---");
                Console.WriteLine(error.Message);
                Console.WriteLine("---Stack Trace---");
                Console.WriteLine(error.StackTrace);
            }
        }
    }
}
