//  2019 ABBYY Development Inc.
// SAMPLES code is property of ABBYY, exclusive rights are reserved. 
//
// DEVELOPER is allowed to incorporate SAMPLES into his own APPLICATION and modify it under 
// the  terms of  License Agreement between  ABBYY and DEVELOPER.


// ABBYY FineReader Engine 12 Sample

using System;
using System.Threading;
using System.Diagnostics;

namespace Sample
{
    class EnginesPool : IDisposable
    {
        public EnginesPool( int enginesCount, string customerProjectId, int waitingEngineTimeout )
        {
            projectId = customerProjectId;
            engineHolders = new EngineHolder [enginesCount];
            for( int i = 0; i < enginesCount; i++ ) {
                engineHolders[i] = new EngineHolder( projectId );
            }

            engineHolderLock = new object();
            waitingTimeout = waitingEngineTimeout;
            semaphore = new Semaphore( enginesCount, enginesCount );
            usageCountLock = new object();
            autoRecycleUsageCount = 0; // Zero means that auto recycling will not be performed
        }

        public FREngine.IEngine GetEngine()
        {
            Trace.Assert( semaphore.WaitOne( waitingTimeout, false ), "Waiting engine timeout exceeded." );

            for( int i = 0; i < engineHolders.Length; i++ ) {
                lock( engineHolderLock ) {
                    if( !engineHolders[i].IsEngineLocked() ) {
                        return engineHolders[i].GetLockedEngine();
                    }
                }
            }
            // The free engine haven't been found
            Trace.Assert( false );
            return null;
        }

        public void ReleaseEngine( FREngine.IEngine engine, bool isRecycleRequired )
        {
            for( int i = 0; i < engineHolders.Length; i++ ) {
                if( engineHolders[i].ContainsEngine( engine ) ) {
                    try {
                        releaseEngine( i, isRecycleRequired );
                    } finally {
                        semaphore.Release();
                    }
                    return;
                }
            }
            // The engine haven't been found
            Trace.Assert( false );
        }

        public int AutoRecycleUsageCount
        {
            get
            {
                lock( usageCountLock ) {
                    return autoRecycleUsageCount;
                }
            }
            set
            {
                lock( usageCountLock ) {
                    autoRecycleUsageCount = value;
                }
            }
        }

        public void Dispose()
        {
            if( engineHolders != null ) {
                for( int i = 0; i < engineHolders.Length; i++ ) {
                    engineHolders[i].Dispose();
                    engineHolders[i] = null;
                }
                engineHolders = null;
            }
        }

        #region Implementation

        private string projectId;
        private EngineHolder[] engineHolders;
        private object engineHolderLock;
        private int waitingTimeout;
        private Semaphore semaphore;
        private object usageCountLock;
        private int autoRecycleUsageCount;

        private void releaseEngine( int engineIndex, bool isRecycleRequired )
        {
            lock( engineHolderLock ) {
                bool isAutoRecycleRequired = AutoRecycleUsageCount != 0 
                    && AutoRecycleUsageCount <= engineHolders[engineIndex].GetEngineUsageCount();

                engineHolders[engineIndex].UnlockEngine();
                if( isRecycleRequired || isAutoRecycleRequired ) {
                    engineHolders[engineIndex].Dispose();
                    engineHolders[engineIndex] = new EngineHolder( projectId );
                }
            }
        }

        class EngineHolder : IDisposable
        {
            public EngineHolder( string projectId )
            {
                try {
                    engineLoader = new FREngine.OutprocLoader();

                    // When a process finishes normally, all COM objects are released and work processes hosting these objects also finish 
                    // normally. If the process terminates abnormally (hard exception or manually killed) the COM objects are not released 
                    // and work processes will remain loaded. To address this issue and make your server more robust you can make 
                    // each work process watch if its parent process is still alive and terminate if not.
                    FREngine.IHostProcessControl processControl = ( FREngine.IHostProcessControl )engineLoader;
                    processControl.SetClientProcessId( Process.GetCurrentProcess().Id );

                    process = Process.GetProcessById( processControl.ProcessId );

                    engine = engineLoader.InitializeEngine( projectId, FreConfig.GetLicensePath(), FreConfig.GetLicensePassword(), "", "", false );
                } catch( System.Runtime.InteropServices.COMException exception ) {
                    uint hResult = ( uint )exception.ErrorCode;
                    if( hResult == 0x80070005 ) {
                        // To use LocalServer under a special account you must add this account to 
                        // the COM-object's launch permissions (using DCOMCNFG or OLE/COM object viewer)
                        throw new Exception( @"Launch permission for the work-process COM-object is not granted.
                            Use DCOMCNFG to change security settings for the object. (" + exception.Message + ")" );
                    }
                    throw;
                }

                isEngineLocked = false;
                engineUsageCount = 0;
            }

            public FREngine.IEngine GetLockedEngine()
            {
                isEngineLocked = true;
                engineUsageCount++;
                return engine;
            }

            public void UnlockEngine()
            {
                isEngineLocked = false;
            }

            public bool IsEngineLocked()
            {
                return isEngineLocked;
            }

            public bool ContainsEngine( FREngine.IEngine value )
            {
                return engine == value;
            }

            public int GetEngineUsageCount()
            {
                return engineUsageCount;
            }

            public void Dispose()
            {
                engine = null;
                GC.Collect();
                GC.WaitForPendingFinalizers();
                if( engineLoader != null ) {
                    engineLoader.ExplicitlyUnload();
                    engineLoader = null;
                    GC.Collect();
                    GC.WaitForPendingFinalizers();
                }

                if( !process.WaitForExit( 5000 ) ) {
                    try {
                        process.Kill();
                    } catch( InvalidOperationException ) {
                        // The process could exit between a decision to kill it and this code
                        // Skip this error
                    }
                }
                process.Dispose();
                process = null;
            }

            private FREngine.IEngineLoader engineLoader;
            private FREngine.IEngine engine;
            private Process process;
            private bool isEngineLocked;
            private int engineUsageCount;
        }

        #endregion
    }
}
