//  2019 ABBYY Development Inc.
// SAMPLES code is property of ABBYY, exclusive rights are reserved. 
//
// DEVELOPER is allowed to incorporate SAMPLES into his own APPLICATION and modify it under 
// the  terms of  License Agreement between  ABBYY and DEVELOPER.


// ABBYY FineReader Engine 12 Sample

using System;
using System.Threading;
using System.Runtime.InteropServices;

namespace Sample
{
    class WorkerThread
    {
        public delegate void TraceHandler( string message );

        public WorkerThread( int id, EnginesPool pool, ImageSource source, TraceHandler handler )
        {
            enginesPool = pool;
            imageSource = source;
            isCancelledLock = new object();
            isCancelled = false;
            traceHandler = handler;
            threadId = id;

            workerThread = new Thread( new ThreadStart( doWork ) );
            workerThread.Priority = ThreadPriority.BelowNormal;
            workerThread.Start();
        }

        public bool IsRunning()
        {
            return workerThread.IsAlive;
        }

        public void Cancel()
        {
            lock( isCancelledLock ) {
                isCancelled = true;
            }
        }

        public bool IsCancelled()
        {
            lock( isCancelledLock ) {
                return isCancelled;
            }
        }

        public void Join()
        {
            workerThread.Join();
        }

        private EnginesPool enginesPool;
        private ImageSource imageSource;
        private Thread workerThread;
        private object isCancelledLock;
        private bool isCancelled;
        private TraceHandler traceHandler;
        private int threadId;

        [MTAThread]
        private void doWork()
        {
            try {
                traceHandler( "Thread " + threadId.ToString() + " started" );

                ImageFile imageFile = imageSource.GetNextImageFile();
                while( !IsCancelled() && imageFile != null ) {
                    traceHandler( "Thread " + threadId.ToString() + ": " + imageFile.FilePath );
                    try {
                        processImageFile( imageFile );
                        imageSource.IncreaseProcessedImageFilesCount();
                    } catch( Exception exception ) {
                        traceHandler( "Thread " + threadId.ToString() + " ERROR: \"" + exception.Message + 
                            "\" during processing image file: " + imageFile.FilePath );
                    }
                    imageFile = imageSource.GetNextImageFile();
                }

                if( !IsCancelled() ) {
                    traceHandler( "Thread " + threadId.ToString() + " finished" );
                } else {
                    traceHandler( "Thread " + threadId.ToString() + " cancelled" );
                }
            } catch( Exception exception ) {
                traceHandler( "Thread " + threadId.ToString() + " ERROR: " + exception.Message );
            }
        }

        private void processImageFile( ImageFile imageFile )
        {
            FREngine.IEngine engine = enginesPool.GetEngine();
            bool isRecycleRequired = false;
            try {
                FREngine.FRDocument frDocument = engine.CreateFRDocument();
                frDocument.OnProgress += new FREngine.DIFRDocumentEvents_OnProgressEventHandler( frDocument_OnProgress );
                try {
                    frDocument.AddImageFile( imageFile.FilePath, null, null );
                    frDocument.Process( null );
                } finally {
                    frDocument.Close();
                }
            } catch( Exception exception ) {
                isRecycleRequired = shouldRestartEngine( exception );
                if( isRecycleRequired ) {
                    imageFile.FailedRunsCount++;
                    if( imageFile.FailedRunsCount > 1 ) {
                        throw;
                    }
                    imageSource.AddImageFile( imageFile );
                } else {
                    throw;
                }
            } finally {
                enginesPool.ReleaseEngine( engine, isRecycleRequired );
            }
        }

        private void frDocument_OnProgress( FREngine.FRDocument Sender, int Percentage, ref bool cancel )
        {
            cancel = IsCancelled();
        }

        private static bool shouldRestartEngine( Exception exception )
        {
            COMException comException = exception as COMException;
            if( comException != null ) {
                uint hResult = ( uint )comException.ErrorCode;
                // The RPC server is unavailable because of a crash of the surrogate process
                // You should restart the engine
                if( hResult == 0x800706BA ) {
                    return true;
                }
                // This is RPC_E_SERVERFAULT because of a structural exception in the surrogate process
                // Most probably now the engine is in the undefined state and you should restart it
                if( hResult == 0x80010105 ) {
                    return true;
                }
            }
            OutOfMemoryException outOfMemoryException = exception as OutOfMemoryException;
            if( outOfMemoryException != null ) {
                return true;
            }
            return false;
        }
    }
}
